import sys
import typing

GenericType = typing.TypeVar("GenericType")


def poly_3d_calc(veclist, pt):
    ''' Calculate barycentric weights for a point on a polygon.

    :param veclist:  list of vectors
    :param pt:  list of per-vector weights
    '''

    pass
